import React from 'react'; //

import { getPrimary, translate } from '../utils/Utils';
import useChartContext from '../utils/chartContext';
export default function Voronoi() {
  var _useChartContext = useChartContext(),
      getOptions = _useChartContext.getOptions,
      focusedDatumState = _useChartContext.focusedDatumState;

  var setFocusedDatum = focusedDatumState[1];

  var _getOptions = getOptions(),
      onFocusDatum = _getOptions.onFocusDatum,
      onClickDatum = _getOptions.onClickDatum,
      tooltip = _getOptions.tooltip,
      primaryCursor = _getOptions.primaryCursor,
      secondaryCursor = _getOptions.secondaryCursor,
      showVoronoi = _getOptions.showVoronoi,
      groupingMode = _getOptions.groupingMode;

  var handleFocus = React.useCallback(function (datum) {
    var _getOptions$onFocusDa, _getOptions2;

    (_getOptions$onFocusDa = (_getOptions2 = getOptions()).onFocusDatum) == null ? void 0 : _getOptions$onFocusDa.call(_getOptions2, datum);
    setFocusedDatum(datum);
  }, [getOptions, setFocusedDatum]);
  var needsVoronoi = onFocusDatum || onClickDatum || tooltip || primaryCursor || secondaryCursor || showVoronoi; // Don't render until we have all dependencies

  if (!needsVoronoi) {
    return null;
  }

  var props = {
    handleFocus: handleFocus
  };

  if (groupingMode === 'primary') {
    return /*#__PURE__*/React.createElement(PrimaryVoronoi, props);
  }

  return null; // return <ClosestVoronoi {...props} />
}

function PrimaryVoronoi(_ref) {
  var handleFocus = _ref.handleFocus;

  var _useChartContext2 = useChartContext(),
      primaryAxis = _useChartContext2.primaryAxis,
      series = _useChartContext2.series,
      secondaryAxes = _useChartContext2.secondaryAxes,
      getOptions = _useChartContext2.getOptions,
      gridDimensions = _useChartContext2.gridDimensions,
      groupedDatums = _useChartContext2.groupedDatums;

  return React.useMemo(function () {
    var columns = series[0].datums.filter(function (datum) {
      var primaryValue = primaryAxis.getValue(datum.originalDatum);
      return primaryValue !== 'undefined' && primaryValue !== null;
    }).map(function (datum, i, all) {
      var _primaryAxis$scale$ra, _groupedDatums$get;

      var prev = all[i - 1];
      var next = all[i + 1];
      var primaryValue = primaryAxis.getValue(datum.originalDatum);
      var primaryPx = getPrimary(datum, primaryAxis);
      var range = (_primaryAxis$scale$ra = primaryAxis == null ? void 0 : primaryAxis.scale.range()) != null ? _primaryAxis$scale$ra : [0, 0];
      var primaryStart = range[0],
          primaryEnd = range[1];

      if (prev) {
        var prevPx = getPrimary(prev, primaryAxis);
        primaryStart = primaryPx - (primaryPx - prevPx) / 2;
      }

      if (next) {
        var nextPx = getPrimary(next, primaryAxis);
        primaryEnd = primaryPx + (nextPx - primaryPx) / 2;
      }

      var datums = (_groupedDatums$get = groupedDatums.get("" + primaryValue)) != null ? _groupedDatums$get : [];
      datums.sort(function (a, b) {
        var _aAxis$scale, _a$stackData, _bAxis$scale, _b$stackData;

        var aAxis = secondaryAxes.find(function (d) {
          return d.id === a.secondaryAxisId;
        });
        var bAxis = secondaryAxes.find(function (d) {
          return d.id === b.secondaryAxisId;
        });
        var aPx = (_aAxis$scale = aAxis == null ? void 0 : aAxis.scale(aAxis.stacked ? (_a$stackData = a.stackData) == null ? void 0 : _a$stackData[1] : aAxis == null ? void 0 : aAxis.getValue(a.originalDatum))) != null ? _aAxis$scale : NaN;
        var bPx = (_bAxis$scale = bAxis == null ? void 0 : bAxis.scale(bAxis.stacked ? (_b$stackData = b.stackData) == null ? void 0 : _b$stackData[1] : bAxis == null ? void 0 : bAxis.getValue(b.originalDatum))) != null ? _bAxis$scale : NaN;
        return aPx - bPx;
      });
      return {
        primaryStart: primaryStart,
        primaryEnd: primaryEnd,
        primaryPx: primaryPx,
        datumBoundaries: datums.filter(function (datum) {
          var secondaryAxis = secondaryAxes.find(function (d) {
            return d.id === datum.secondaryAxisId;
          });
          var secondaryValue = secondaryAxis == null ? void 0 : secondaryAxis.getValue(datum.originalDatum);
          return typeof secondaryValue !== 'undefined' && secondaryValue !== null;
        }).map(function (datum, i, all) {
          var _secondaryAxis$scale3, _secondaryAxis$scale$2;

          var prev = all[i - 1];
          var next = all[i + 1];
          var secondaryAxis = secondaryAxes.find(function (d) {
            return d.id === datum.secondaryAxisId;
          });

          if (secondaryAxis != null && secondaryAxis.stacked) {
            var _secondaryAxis$scale$, _datum$stackData, _datum$stackData2;

            var _range = (_secondaryAxis$scale$ = secondaryAxis == null ? void 0 : secondaryAxis.scale.range()) != null ? _secondaryAxis$scale$ : [0, 0];

            var stackData = [(_datum$stackData = datum.stackData) == null ? void 0 : _datum$stackData[0], (_datum$stackData2 = datum.stackData) == null ? void 0 : _datum$stackData2[1]];

            if (secondaryAxis != null && secondaryAxis.isVertical) {
              _range.reverse();

              stackData.reverse();
            }

            var _secondaryStart = _range[0],
                _secondaryEnd = _range[1];

            if (prev) {
              var _secondaryAxis$scale, _stackData$;

              _secondaryStart = (_secondaryAxis$scale = secondaryAxis == null ? void 0 : secondaryAxis.scale((_stackData$ = stackData[0]) != null ? _stackData$ : NaN)) != null ? _secondaryAxis$scale : NaN;
            }

            if (next) {
              var _secondaryAxis$scale2, _stackData$2;

              _secondaryEnd = (_secondaryAxis$scale2 = secondaryAxis == null ? void 0 : secondaryAxis.scale((_stackData$2 = stackData[1]) != null ? _stackData$2 : NaN)) != null ? _secondaryAxis$scale2 : NaN;
            }

            return {
              secondaryStart: _secondaryStart,
              secondaryEnd: _secondaryEnd,
              datum: datum
            };
          }

          var value = (_secondaryAxis$scale3 = secondaryAxis == null ? void 0 : secondaryAxis.scale(secondaryAxis == null ? void 0 : secondaryAxis.getValue(datum.originalDatum))) != null ? _secondaryAxis$scale3 : NaN;
          var range = (_secondaryAxis$scale$2 = secondaryAxis == null ? void 0 : secondaryAxis.scale.range()) != null ? _secondaryAxis$scale$2 : [0, 0];

          if (secondaryAxis != null && secondaryAxis.isVertical) {
            range.reverse();
          }

          var secondaryStart = range[0],
              secondaryEnd = range[1];

          if (prev) {
            var _prevAxis$scale;

            var prevAxis = secondaryAxes.find(function (d) {
              return d.id === (prev == null ? void 0 : prev.secondaryAxisId);
            });
            var prevValue = (_prevAxis$scale = prevAxis == null ? void 0 : prevAxis.scale(prevAxis == null ? void 0 : prevAxis.getValue(prev.originalDatum))) != null ? _prevAxis$scale : NaN;
            secondaryStart = value - (value - prevValue) / 2;
          }

          if (next) {
            var _nextAxis$scale;

            var nextAxis = secondaryAxes.find(function (d) {
              return d.id === (next == null ? void 0 : next.secondaryAxisId);
            });
            var nextValue = (_nextAxis$scale = nextAxis == null ? void 0 : nextAxis.scale(nextAxis == null ? void 0 : nextAxis.getValue(next.originalDatum))) != null ? _nextAxis$scale : NaN;
            secondaryEnd = value + (nextValue - value) / 2;
          }

          return {
            secondaryStart: secondaryStart,
            secondaryEnd: secondaryEnd,
            datum: datum
          };
        })
      };
    });
    return /*#__PURE__*/React.createElement("g", {
      onMouseLeave: function onMouseLeave() {
        return handleFocus(null);
      },
      style: {
        transform: translate(gridDimensions.left, gridDimensions.top)
      }
    }, columns.map(function (column) {
      return /*#__PURE__*/React.createElement(React.Fragment, {
        key: column.primaryPx
      }, column.datumBoundaries.map(function (datumBoundary) {
        var x1 = !primaryAxis.isVertical ? column.primaryStart : datumBoundary.secondaryStart;
        var x2 = !primaryAxis.isVertical ? column.primaryEnd : datumBoundary.secondaryEnd;
        var y1 = !primaryAxis.isVertical ? datumBoundary.secondaryStart : column.primaryStart;
        var y2 = !primaryAxis.isVertical ? datumBoundary.secondaryEnd : column.primaryEnd;
        var x = Math.min(x1, x2);
        var y = Math.min(y1, y2);
        var xEnd = Math.max(x1, x2);
        var yEnd = Math.max(y1, y2);
        var height = Math.max(yEnd - y, 0);
        var width = Math.max(xEnd - x, 0);
        return /*#__PURE__*/React.createElement("rect", {
          key: column.primaryPx + "_" + datumBoundary.datum.seriesIndex,
          x: x,
          y: y,
          width: width,
          height: height,
          className: 'action-voronoi',
          onMouseEnter: function onMouseEnter() {
            return handleFocus(datumBoundary.datum);
          },
          style: {
            fill: getOptions().dark ? '#ffffff33' : 'rgba(0,0,0,0.2)',
            strokeWidth: 1,
            stroke: getOptions().dark ? 'white' : 'black',
            opacity: getOptions().showVoronoi ? 1 : 0
          }
        });
      }));
    }));
  }, [getOptions, gridDimensions.left, gridDimensions.top, groupedDatums, handleFocus, primaryAxis, secondaryAxes, series]);
} // function ClosestVoronoi({
//   stackData,
//   extent,
//   handleFocus,
//   showVoronoi,
// }: {
//   stackData: Series[]
//   extent: number[][]
//   handleFocus: (datum: Datum | null) => void
//   showVoronoi: boolean
// }) {
//   let polygons = null
//   const voronoiData: { x: number; y: number; datum: Datum }[] = []
//   stackData.forEach(series => {
//     series.datums
//       .filter(d => d.defined)
//       .forEach(datum => {
//         datum.boundingPoints.forEach(boundingPoint => {
//           if (
//             typeof datum.x !== 'number' ||
//             typeof datum.y !== 'number' ||
//             Number.isNaN(datum.y) ||
//             Number.isNaN(datum.x)
//           ) {
//             return
//           }
//           voronoiData.push({
//             x: boundingPoint.x,
//             y: boundingPoint.y,
//             datum,
//           })
//         })
//       })
//   })
//   const delaunay = Delaunay.from(
//     voronoiData,
//     d => Math.max(d.x, 0),
//     d => Math.max(d.y, 0)
//   )
//   const flatExtent = extent.flat().map(d => Math.max(d, 0))
//   const voronoi = delaunay.voronoi(flatExtent)
//   polygons = voronoi.cellPolygons()
//   polygons = Array.from(polygons)
//   return (
//     <g>
//       {polygons.map((points, i) => {
//         const index = points.index
//         const datum = voronoiData[index].datum
//         const path = lineFn(points as any) || undefined
//         return (
//           <Path
//             key={i}
//             d={path}
//             className="action-voronoi"
//             onMouseEnter={() => handleFocus(datum)}
//             onMouseLeave={() => handleFocus(null)}
//             style={{
//               fill: randomFill(),
//               opacity: showVoronoi ? 1 : 0,
//             }}
//           />
//         )
//       })}
//     </g>
//   )
// }