import _extends from "@babel/runtime/helpers/esm/extends";
export function getSeriesStatus(series, focusedDatum) {
  if ((focusedDatum == null ? void 0 : focusedDatum.seriesId) === series.id) {
    return 'focused';
  }

  return 'none';
}
export function getDatumStatus(datum, focusedDatum) {
  var _datum$group;

  if (datum === focusedDatum) {
    return 'focused';
  }

  if ((_datum$group = datum.group) != null && _datum$group.some(function (groupDatum) {
    groupDatum.seriesId === (focusedDatum == null ? void 0 : focusedDatum.seriesId) && groupDatum.index === (focusedDatum == null ? void 0 : focusedDatum.index);
  })) {
    return 'groupFocused';
  }

  return 'none';
}

function normalizeColor(style, defaults) {
  return _extends({}, style, {
    stroke: style.stroke || style.color || defaults.stroke || defaults.color,
    fill: style.fill || style.color || defaults.fill || defaults.color
  });
}

var elementTypes = ['area', 'line', 'rectangle', 'circle'];
export function materializeStyles(style, defaults) {
  if (style === void 0) {
    style = {};
  }

  if (defaults === void 0) {
    defaults = {};
  }

  style = normalizeColor(style, defaults);

  for (var i = 0; i < elementTypes.length; i++) {
    var type = elementTypes[i];

    if (style[type] && defaults[type]) {
      style[type] = materializeStyles(style[type], defaults);
    }
  }

  return style;
}
export function isValidPoint(d) {
  if (d === null) {
    return false;
  }

  if (typeof d === 'undefined') {
    return false;
  }

  if (typeof d === 'string' && d === 'null') {
    return false;
  }

  return true;
}
export function translateX(x) {
  return "translate3d(" + Math.round(x) + "px, 0, 0)";
}
export function translateY(y) {
  return "translate3d(0, " + Math.round(y) + "px, 0)";
}
export function translate(x, y) {
  return "translate3d(" + Math.round(x) + "px, " + Math.round(y) + "px, 0)";
}
export function getSecondaryStart(datum, secondaryAxis) {
  var _secondaryAxis$scale2;

  if (secondaryAxis.stacked) {
    var _secondaryAxis$scale, _datum$stackData$, _datum$stackData;

    return (_secondaryAxis$scale = secondaryAxis.scale((_datum$stackData$ = (_datum$stackData = datum.stackData) == null ? void 0 : _datum$stackData[0]) != null ? _datum$stackData$ : NaN)) != null ? _secondaryAxis$scale : NaN;
  }

  return (_secondaryAxis$scale2 = secondaryAxis.scale(0)) != null ? _secondaryAxis$scale2 : NaN;
}
export function getSecondary(datum, secondaryAxis) {
  var _secondaryAxis$scale4;

  if (secondaryAxis.stacked) {
    var _secondaryAxis$scale3, _datum$stackData$2, _datum$stackData2;

    return (_secondaryAxis$scale3 = secondaryAxis.scale((_datum$stackData$2 = (_datum$stackData2 = datum.stackData) == null ? void 0 : _datum$stackData2[1]) != null ? _datum$stackData$2 : NaN)) != null ? _secondaryAxis$scale3 : NaN;
  }

  return (_secondaryAxis$scale4 = secondaryAxis.scale(secondaryAxis.getValue(datum.originalDatum))) != null ? _secondaryAxis$scale4 : NaN;
}
export function getPrimary(datum, primaryAxis) {
  var primary;

  if (primaryAxis.stacked) {
    var _primaryAxis$scale, _datum$stackData$3, _datum$stackData3;

    primary = (_primaryAxis$scale = primaryAxis.scale((_datum$stackData$3 = (_datum$stackData3 = datum.stackData) == null ? void 0 : _datum$stackData3[1]) != null ? _datum$stackData$3 : NaN)) != null ? _primaryAxis$scale : NaN;
  } else {
    var _primaryAxis$scale2;

    primary = (_primaryAxis$scale2 = primaryAxis.scale(primaryAxis.getValue(datum.originalDatum))) != null ? _primaryAxis$scale2 : NaN;
  }

  if (primaryAxis.axisFamily === 'band') {
    primary = primary + getPrimaryLength(datum, primaryAxis) / 2;
  }

  return primary;
}
export function getPrimaryStart(datum, primaryAxis) {
  return getPrimary(datum, primaryAxis) - getPrimaryLength(datum, primaryAxis) / 2;
}
export function getPrimaryLength(_datum, primaryAxis) {
  if (primaryAxis.axisFamily === 'band') {
    var _primaryAxis$minBandS, _primaryAxis$maxBandS;

    return Math.min(Math.max(primaryAxis.scale.bandwidth(), (_primaryAxis$minBandS = primaryAxis.minBandSize) != null ? _primaryAxis$minBandS : 1), (_primaryAxis$maxBandS = primaryAxis.maxBandSize) != null ? _primaryAxis$maxBandS : 99999999);
  }

  return Math.max(primaryAxis.bandScale.bandwidth(), 1);
}
export function getSecondaryLength(datum, secondaryAxis) {
  var secondary = [getSecondaryStart(datum, secondaryAxis), getSecondary(datum, secondaryAxis)];
  return Math.abs(secondary[1] - secondary[0]);
}
export function getX(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getSecondary(datum, secondaryAxis) : getPrimary(datum, primaryAxis);
}
export function getXStart(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getSecondaryStart(datum, secondaryAxis) : getPrimaryStart(datum, primaryAxis);
}
export function getY(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getPrimary(datum, primaryAxis) : getSecondary(datum, secondaryAxis);
}
export function getYStart(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getPrimaryStart(datum, primaryAxis) : getSecondaryStart(datum, secondaryAxis);
}
export function getRectX(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getSecondaryStart(datum, secondaryAxis) : getPrimaryStart(datum, primaryAxis);
}
export function getRectY(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getPrimaryStart(datum, primaryAxis) : getSecondary(datum, secondaryAxis);
}
export function getWidth(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getSecondaryLength(datum, secondaryAxis) : getPrimaryLength(datum, primaryAxis);
}
export function getHeight(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getPrimaryLength(datum, primaryAxis) : getSecondaryLength(datum, secondaryAxis);
}
export function clampPxToAxis(px, axis) {
  var range = axis.scale.range();

  if (axis.isVertical) {
    range.reverse();
  }

  return Math.max(range[0], Math.min(px, range[1]));
}
export function getTickPx(scale, value) {
  var _scale;

  var px = (_scale = scale(value)) != null ? _scale : NaN; // @ts-ignore

  if (scale.bandwidth) {
    // @ts-ignore
    return px + scale.bandwidth() / 2;
  }

  return px;
}